<?php

namespace Drupal\drupalorg\Controller;

use Drupal\contribution_records\SourceLink\DevGitDrupalCodeIssue;
use Drupal\contribution_records\SourceLink\GitDrupalCodeBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Queue\QueueFactory;
use Drupal\drupalorg\Traits\GitLabClientTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller to manage issue forks, merge requests, etc.
 */
class IssueForksController extends ControllerBase {

  use GitLabClientTrait;

  /**
   * Prefix to follow on fork names for retrieval and creation.
   *
   * @var string
   */
  const FORK_NAME_PREFIX = 'drupalorg-';

  /**
   * Queue factory service.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected QueueFactory $queueFactory;

  /**
   * Config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $config;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('queue')
    );
  }

  /**
   * Construct method.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   Queue factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory, QueueFactory $queue_factory) {
    $this->config = $config_factory;
    $this->queueFactory = $queue_factory;
  }

  /**
   * Check access to an issue fork.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Whether the user has access to the fork or not in JSON format.
   */
  public function issueForkCheckAccess(Request $request): JsonResponse {
    $data = [
      'access' => FALSE,
    ];

    $fork_id_param = $request->query->get('fork_id');
    if (empty($fork_id_param)) {
      $data['error'] = '"fork_id" is required';
      return new JsonResponse($data);
    }

    $gitlab_user_id = $this->getGitLabUserIdFromUserId($this->currentUser()->id());
    if (empty($gitlab_user_id)) {
      $data['error'] = 'You need to set up your git username in your profile';
      return new JsonResponse($data);
    }

    $data['fork_id'] = $fork_id_param;
    $data['gitlab_user_id'] = $gitlab_user_id;

    try {
      $member = $this->getGitLabClient()->projects()->member($fork_id_param, $gitlab_user_id);
      $data['access'] = !empty($member);
    }
    catch (\Throwable $e) {
      $data['message'] = 'Not allowed or fork not found';
      $this->getLogger('drupalorg')->error('Error checking member in fork. Message: @message', [
        '@message' => $e->getMessage(),
      ]);
    }

    return new JsonResponse($data);
  }

  /**
   * Grant access to an issue fork.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   Render array or redirect.
   */
  public function issueForkRequestAccess(Request $request): RedirectResponse | array {
    $fork_id_param = $request->query->get('fork_id');
    $source_link_param = $request->query->get('source_link');
    [
      'project' => $project_id,
      'issue_iid' => $issue_iid,
      'error' => $error,
    ] = $this->getProjectAndIssueIdFromUrl($source_link_param);

    if (!empty($error)) {
      $this->messenger()->addError($error);
      return [
        '#title' => $this->t('Forks management'),
        '#markup' => $this->t('Something went wrong.'),
      ];
    }

    $this->queueFactory->get('drupalorg_issue_forks_queue_worker')->createItem([
      'action' => 'request_access',
      'fork_id' => $fork_id_param,
      'user_id' => $this->currentUser()->id(),
      'issue_id' => $issue_iid,
    ]);
    $this->messenger()->addStatus($this->t('Your request was queued.'));

    return $this->redirect('drupalorg.issue_fork_management', [], [
      'query' => [
        'source_link' => $source_link_param,
      ],
    ]);
  }

  /**
   * Creates a new fork.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   Render array or redirect.
   */
  public function issueForkCreateFork(Request $request): RedirectResponse | array {
    $fork_name_param = $request->query->get('fork_name');
    $source_link_param = $request->query->get('source_link');
    [
      'project' => $project_id,
      'issue_iid' => $issue_iid,
      'error' => $error,
    ] = $this->getProjectAndIssueIdFromUrl($source_link_param);

    if (!empty($error)) {
      $this->messenger()->addError($error);
      return [
        '#title' => $this->t('Forks management'),
        '#markup' => $this->t('Something went wrong.'),
      ];
    }

    $fork_created = NULL;
    $client = $this->getGitLabClient();
    try {
      $fork_name = $fork_name_param;
      if (empty($fork_name)) {
        $fork_name = self::FORK_NAME_PREFIX . $issue_iid;
      }
      $fork = $client->projects()->fork($project_id, [
        'namespace' => 'issue',
        'path' => $fork_name,
        'name' => $fork_name,
      ]);
      if (!empty($fork['name'])) {
        // This part is not relevant to creating the fork, so queue it.
        $this->queueFactory->get('drupalorg_issue_forks_queue_worker')->createItem([
          'action' => 'post_fork_creation',
          'fork_id' => $fork['id'],
          'user_id' => $this->currentUser()->id(),
          'issue_id' => $issue_iid,
        ]);

        $this->messenger()->addStatus($this->t('The fork "@fork" was created successfully.', [
          '@fork' => $fork['name'],
        ]));
        $fork_created = $fork['name'];
      }
      else {
        $this->messenger()->addWarning($this->t('Creating the fork failed.'));
      }
    }
    catch (\Throwable $e) {
      $this->messenger()->addError($this->t('Could not create fork.'));
      $this->getLogger('drupalorg')->error('Error creating fork. Message: @message', [
        '@message' => $e->getMessage(),
      ]);
    }

    return $this->redirect('drupalorg.issue_fork_management', [], [
      'query' => [
        'source_link' => $source_link_param,
        'fork_created' => $fork_created ?? '',
      ],
    ]);
  }

  /**
   * Page to manage all issue forks, MRs, access, etc.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return array
   *   Render array.
   */
  public function issueForksManagement(Request $request): array {
    $source_link_param = $request->query->get('source_link');
    [
      'project' => $project_id,
      'issue_iid' => $issue_iid,
      'error' => $error,
    ] = $this->getProjectAndIssueIdFromUrl($source_link_param);

    if (!empty($error)) {
      $this->messenger()->addError($error);
      return [
        '#title' => $this->t('Forks management'),
        '#markup' => $this->t('Something went wrong.'),
      ];
    }

    $client = $this->getGitLabClient();
    $issue = NULL;
    $forks = NULL;
    $merge_requests = NULL;
    $project = NULL;
    try {
      $project = $client->projects()->show($project_id);
      $issue = $client->issues()->show($project_id, $issue_iid);
      $merge_requests = $client->issues()->relatedMergeRequests($project_id, $issue_iid);
      $forks = $client->projects()->forks($project_id, [
        // @todo Maybe adding quotes will do exact search, otherwise we will
        // get prefix-1, prefix-10... for issue 1.
        'search' => self::FORK_NAME_PREFIX . $issue_iid,
      ]);
    }
    catch (\Throwable $e) {
      // Let the front-end display that there is no data.
      $this->getLogger('drupalorg')->error('Error getting GitLab info. Message: @message', [
        '@message' => $e->getMessage(),
      ]);
    }

    if (empty($issue)) {
      $this->messenger()->addWarning($this->t('There was a problem finding the issue. Please make sure that the link provided is valid.'));
    }

    return [
      '#title' => $issue['title'] ?? $this->t('Not found'),
      '#theme' => 'drupalorg_issue_forks_management',
      '#forks' => $forks,
      '#fork_prefix' => self::FORK_NAME_PREFIX,
      '#merge_requests' => $merge_requests,
      '#issue' => $issue,
      '#project' => $project,
      '#logged_in' => $this->currentUser()->isAuthenticated(),
      '#cache' => [
        'max-age' => 60,
        'contexts' => [
          'url',
        ],
      ],
    ];
  }

  /**
   * Parses the URL and extracts the project and issue id from it.
   *
   * @param string|null $source_link_param
   *   Link to parse.
   *
   * @return array
   *   Array containing the project and the issue id.
   */
  protected function getProjectAndIssueIdFromUrl(?string $source_link_param = NULL): array {
    $info = [
      'project' => NULL,
      'issue_iid' => NULL,
      'error' => NULL,
    ];

    if (empty($source_link_param)) {
      $info['error'] = $this->t('"source_link" parameter is required.');
      return $info;
    }

    // Format: https://git.drupalcode.org/project/config_notify/-/issues/10
    $url_parts = parse_url($source_link_param);

    // Only Gitlab issues have the fork management done via this page. If the
    // source link is an MR, that means that the forking and branching is taken
    // care of, and for issues coming from www.drupal.org, the fork management
    // is still done via the issue page.
    $accepted_domains = [GitDrupalCodeBase::DOMAIN];
    if ($this->config->get('contribution_records.settings')->get('allow_dev_sources')) {
      $accepted_domains[] = DevGitDrupalCodeIssue::DOMAIN;
    }
    if (!in_array($url_parts['host'], $accepted_domains)) {
      $info['error'] = $this->t('"source_link" has an invalid domain. Only issues from the following domains are accepted: @domains.', [
        '@domains' => implode(', ', $accepted_domains),
      ]);
      return $info;
    }

    if (
      empty($url_parts['host']) ||
      empty($url_parts['scheme']) ||
      empty($url_parts['path']) ||
      !str_starts_with($url_parts['path'], '/project/') ||
      !str_contains($url_parts['path'], '-/issues/')
    ) {
      $info['error'] = $this->t('"source_link" is not a valid link.');
      return $info;
    }

    // URL seems valid, extra the project name and the issue id.
    $path = explode('/', trim($url_parts['path'], '/'));
    if (
      count($path) !== 5 ||
      $path[0] !== 'project' ||
      $path[2] !== '-' ||
      $path[3] !== 'issues' ||
      !is_numeric($path[4])
    ) {
      $info['error'] = $this->t('"source_link" is not a valid link.');
      return $info;
    }

    // We know for sure that the URL is well formatted.
    $info['project'] = 'project/' . $path[1];
    $info['issue_iid'] = (int) $path[4];

    return $info;
  }

}
