<?php

namespace Drupal\drupalorg\Drush\Commands;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\MemoryCache\MemoryCache;
use Drupal\drupalorg\ProjectService;
use Drupal\drupalorg\Traits\GitLabClientTrait;
use Drupal\drupalorg\Utilities\ActiveInstalls;
use Drupal\drupalorg\Utilities\ComposerNamespace;
use Drupal\drupalorg\Utilities\CoreCompatibility;
use Drupal\node\Entity\Node;
use Drush\Commands\AutowireTrait;
use Drush\Commands\DrushCommands as BaseDrushCommands;
use Drush\Attributes as CLI;
use Drush\Drush;
use Gitlab\ResultPager;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * A drush command file.
 *
 * @package Drupal\drupalorg\Commands
 */
final class DrushCommands extends BaseDrushCommands {

  use AutowireTrait;
  use GitLabClientTrait;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    #[Autowire(service: 'entity.memory_cache')]
    protected MemoryCache $memoryCache,
    #[Autowire(service: 'drupalorg.project_service')]
    protected ProjectService $projectService,
    protected TimeInterface $time,
  ) {
    parent::__construct();
  }

  /**
   * Calculate the active installs for modules based on the usage table.
   */
  #[CLI\Command(name: 'drupalorg:calculate-active-installs')]
  public function calculateActiveInstalls() {
    ActiveInstalls::calculateNewActiveInstalls(NULL, Drush::verbose());
  }

  /**
   * Calculate the composer namespaces for modules based on update status info.
   *
   * @see https://www.drupal.org/drupalorg/docs/apis/update-status-xml
   */
  #[CLI\Command(name: 'drupalorg:calculate-composer-namespaces')]
  public function calculateComposerNamespaces() {
    // This is setting a default "drupal/<machine_name>" if not found. Due
    // to D7 and below modules. Otherwise, it'll keep trying those every time.
    // The below line could be part of an update hook.
    ComposerNamespace::calculateNamespace(TRUE);
  }

  /**
   * Calculate the composer compatibility for modules based on their releases.
   */
  #[CLI\Command(name: 'drupalorg:calculate-composer-compatibilities')]
  public function calculateComposerCompatibilities() {
    $last_run = \Drupal::state()->get('drupalorg.composer_compatibilities');
    $request_time = \Drupal::time()->getRequestTime();
    CoreCompatibility::calculateNewCoreCompatibilities($last_run);
    \Drupal::state()->set('drupalorg.composer_compatibilities', $request_time);
  }

  /**
   * Check for disabled GitLab system hooks.
   */
  #[CLI\Command(name: 'drupalorg:check-gitlab-system-hooks')]
  public function checkGitlabSystemHooks(): int {
    $return = self::EXIT_SUCCESS;
    foreach ($this->getGitLabClient()->systemHooks()->all() as $hook) {
      $this->logger->notice('{url} is {alert_status}, disabled until {disabled_until}', [
        'url' => $hook['url'],
        'alert_status' => $hook['alert_status'],
        'disabled_until' => $hook['disabled_until'],
      ]);
      if ($hook['alert_status'] !== 'executable') {
        $this->logger->error('{url} system hook is not executable! API data: {data}', [
          'url' => $hook['url'],
          'data' => json_encode($hook),
        ]);
        $return = self::EXIT_FAILURE_WITH_CLARITY;
      }
    }

    if ($return === self::EXIT_SUCCESS) {
      $this->logger->notice('GitLab system hooks are all enabled!');
    }

    return $return;
  }

  /**
   * Fetch GitLab avatars and set them as logos where available.
   */
  #[CLI\Command(name: 'drupalorg:fetch-gitlab-avatars')]
  public function fetchGitlabAvatars(): void {
    try {
      $client = $this->getGitLabClient();

      $project_nids = \Drupal::entityQuery('node')
        ->accessCheck(FALSE)
        ->condition('type', ProjectService::PROJECT_TYPES, 'IN')
        ->notExists('field_logo_url')
        ->execute();
      foreach (array_chunk($project_nids, 100) as $nids) {
        foreach (Node::loadMultiple($nids) as $project_node) {
          $repository = $this->projectService->getProjectRepositoryInformation($project_node);
          if (empty($repository['gitlab_project_id'])) {
            continue;
          }

          $this->logger->notice('Checking project logo for {project}', [
            'project' => $project_node->get('field_project_machine_name')->value,
          ]);

          $gitlab_project = $client->projects()->show($repository['gitlab_project_id']);
          $this->projectService->updateLogo($project_node, $gitlab_project['avatar_url'] ?? '');
        }
        $this->memoryCache->deleteAll();
      }
    }
    catch (\Throwable $e) {
      $this->logger->error('Could not connect to GitLab or fetch avatars. Code {code} at {at}. Message: {message}', [
        'code' => $e->getCode(),
        'at' => $e->getFile() . ':' . $e->getLine(),
        'message' => $e->getMessage(),
      ]);
    }
  }

  /**
   * Populate milestones for security team.
   */
  #[CLI\Command(name: 'drupalorg:security-populate-milestones')]
  public function securityPopulateMilestones() {
    $gitlab_client = $this->getGitLabClient();
    $milestones = array_column((new ResultPager($gitlab_client))->fetchAll($gitlab_client->groupsMilestones(), 'all', ['security']), 'id', 'due_date');

    $wednesdays = new \DatePeriod(new \DateTimeImmutable('Wednesday'), new \DateInterval('P7D'), 16);
    foreach ($wednesdays as $wednesday) {
      $date = $wednesday->format('Y-m-d');
      if (!isset($milestones[$date])) {
        $label = $date;
        $day_of_month = (int) $wednesday->format('j');
        if ($day_of_month > 14 && $day_of_month <= 21) {
          $label .= ' core';
        }
        $gitlab_client->groupsMilestones()->create('security', [
          'start_date' => $wednesday->sub(new \DateInterval('P6D'))->format('Y-m-d'),
          'due_date' => $date,
          'title' => $label,
        ]);
      }
    }
  }

}
