<?php

namespace Drupal\drupalorg\Plugin\QueueWorker;

use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\drupalorg\Traits\GitLabClientTrait;

/**
 * Defines 'drupalorg_issue_forks_queue_worker' queue worker.
 *
 * Run via `drush` like this:
 * `drush queue:run drupalorg_issue_forks_queue_worker`.
 *
 * @QueueWorker(
 *   id = "drupalorg_issue_forks_queue_worker",
 *   title = @Translation("Issue Forks Queue Worker")
 * )
 */
class DrupalOrgIssueForksQueueWorker extends QueueWorkerBase {

  use GitLabClientTrait;

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    if (!$this->validateItem($data)) {
      return FALSE;
    }

    switch ($data['action']) {
      case 'request_access':
        $this->requestAccess($data);
        break;

      case 'post_fork_creation':
        $this->postForkCreation($data);
        break;
    }
  }

  /**
   * Validate item array to make sure all key elements are there.
   *
   * @param array $data
   *   Item to validate.
   *
   * @return bool
   *   Whether the item was valid or not.
   */
  protected function validateItem(array $data) {
    if (
      empty($data['action']) ||
      empty($data['fork_id']) ||
      empty($data['issue_id']) ||
      empty($data['user_id'])
    ) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Gives the user access to the fork.
   *
   * @param array $data
   *   Data given to the queue containing user and fork information.
   */
  protected function requestAccess(array $data) {
    $client = $this->getGitLabClient();
    $gitlab_user_id = $this->getGitLabUserIdFromUserId($data['user_id']);
    if (!empty($gitlab_user_id)) {
      try {
        // 30 is “Developer access” https://docs.gitlab.com/ee/api/members.html
        $access_level = ($gitlab_user_id == 1) ? '50' : '30';
        $client->projects()->addMember($data['fork_id'], $gitlab_user_id, $access_level);
      }
      catch (\Throwable $e) {
        if ($e->getMessage() !== 'Member already exists') {
          \Drupal::logger('drupalorg')->error('Error adding member to fork. Message: @message', [
            '@message' => $e->getMessage(),
          ]);
        }
      }
    }
  }

  /**
   * Creates a branch based on the issue id and adds the user as member.
   *
   * @param array $data
   *   Data given to the queue containing user, issue and fork information.
   */
  protected function postForkCreation(array $data) {
    $client = $this->getGitLabClient();

    try {
      $fork = $client->projects()->show($data['fork_id']);

      // Create a branch with the issue number in it.
      if (!empty($fork['forked_from_project']['default_branch'])) {
        $branch_name = 'issue-' . $data['issue_id'] . '-branch';
        $client->repositories()->createBranch($data['fork_id'], $branch_name, $fork['forked_from_project']['default_branch']);
      }

      // Add the user that created the fork as member.
      $gitlab_user_id = $this->getGitLabUserIdFromUserId($data['user_id']);
      if ($gitlab_user_id) {
        $access_level = ($gitlab_user_id == 1) ? '50' : '30';
        $client->projects()->addMember($fork['id'], $gitlab_user_id, $access_level);
      }
    }
    catch (\Throwable $e) {
      \Drupal::logger('drupalorg')->error('Error in post fork creation. Message: @message', [
        '@message' => $e->getMessage(),
      ]);
    }
  }

}
