<?php

namespace Drupal\drupalorg;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\node\NodeInterface;
use Psr\Log\LoggerInterface;

/**
 * Project related helper methods.
 */
class ProjectService {

  /**
   * Types of projects.
   *
   * This comes from "project_project_node_types" result in D7.
   *
   * @var array
   */
  const PROJECT_TYPES = [
    'project_module',
    'project_theme',
    'project_distribution',
    'project_theme_engine',
    'project_general',
    'project_drupalorg',
    'project_translation',
    'project_core',
  ];

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $connection;

  /**
   * Construct method.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   */
  public function __construct(Connection $connection, protected EntityTypeManagerInterface $entityTypeManager, protected LoggerInterface $logger) {
    $this->connection = $connection;
  }

  /**
   * Retrieve a project given a value and a field.
   *
   * @param string $field
   *   Field to query.
   * @param string $value
   *   Value to check.
   * @param string|null $type
   *   Type of project node to query, otherwise all project types.
   *
   * @return \Drupal\node\NodeInterface|null
   *   Project node or null.
   */
  protected function getProjectBy(string $field, string $value, ?string $type = NULL): ?NodeInterface {
    $project_types = is_null($type) ? self::PROJECT_TYPES : [$type];
    $project_id = $this->entityTypeManager->getStorage('node')->getQuery()
      ->accessCheck(FALSE)
      ->condition('type', $project_types, 'IN')
      ->condition($field, $value)
      ->range(0, 1)
      ->execute();
    if (!empty($project_id)) {
      return $this->entityTypeManager->getStorage('node')->load(reset($project_id));
    }

    return NULL;
  }

  /**
   * Retrieve a project from its composer username.
   *
   * @param string $composer_namespace
   *   Composer namespace of the project.
   *
   * @return \Drupal\node\NodeInterface|null
   *   Project node or null.
   */
  public function getProjectByComposerNamespace(string $composer_namespace): ?NodeInterface {
    return $this->getProjectBy('field_composer_namespace', $composer_namespace, 'project_module');
  }

  /**
   * Retrieve a project from its machine name.
   *
   * @param string $machine_name
   *   Machine name of the project.
   *
   * @return \Drupal\node\NodeInterface|null
   *   Project node or null.
   */
  public function getProjectByMachineName(string $machine_name): ?NodeInterface {
    return $this->getProjectBy('field_project_machine_name', $machine_name);
  }

  /**
   * Get the maintainers of a given project.
   *
   * @param \Drupal\node\NodeInterface $project
   *   Project to get maintainers from.
   *
   * @return array
   *   List of users that are maintainers of the project.
   */
  public function getProjectMaintainers(NodeInterface $project): array {
    if (!$this->isProject($project)) {
      throw new \Exception('The entity is not a project.');
    }

    $query = $this->connection
      ->select('project_maintainer', 'pm')
      ->condition('pm.nid', $project->id())
      ->fields('pm', ['uid']);
    $maintainers = $query->execute()->fetchCol();
    if (!empty($maintainers)) {
      $maintainers = $this->entityTypeManager->getStorage('user')->loadMultiple($maintainers);
    }

    return $maintainers;
  }

  /**
   * Return whether a given user is a maintainer of a project node.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   User to check.
   * @param \Drupal\node\NodeInterface $node
   *   Node to check.
   *
   * @return bool
   *   Whether the user is a maintainer of the project.
   */
  public function isProjectMaintainer(AccountInterface $account, NodeInterface $node) {
    $maintainers = $this->getProjectMaintainers($node);
    return !empty($maintainers[$account->id()]);
  }

  /**
   * Get the repository information of a given project.
   *
   * @param \Drupal\node\NodeInterface $project
   *   Project to get repository information from.
   *
   * @return array
   *   Repository namespace, name and id.
   */
  public function getProjectRepositoryInformation(NodeInterface $project): array {
    $query = $this->connection
      ->select('drupalorg_project_repositories', 'pr')
      ->condition('pr.drupal_project_nid', $project->id())
      ->fields('pr');
    $info = $query->execute()->fetchAssoc();

    return [
      'drupal_project_nid' => $info['drupal_project_nid'] ?? NULL,
      'gitlab_project_id' => $info['gitlab_project_id'] ?? NULL,
      'gitlab_namespace' => $info['gitlab_namespace'] ?? NULL,
      'gitlab_project_name' => $info['gitlab_project_name'] ?? NULL,
    ];
  }

  /**
   * Get a Drupal project from the GitLab path.
   *
   * @param string $path
   *   Path within GitLab.
   *
   *   It can be a GitLab project ID, the project name, or the
   *   namespace/project_name.
   *
   * @return \Drupal\node\NodeInterface|null
   *   Project node or null.
   */
  public function getProjectByRepositoryPath($path): ?NodeInterface {
    $gitlab_project_id = is_numeric($path) ? (int) $path : NULL;
    $namespace = 'project';
    $name = !is_numeric($path) ? $path : NULL;
    if (str_contains($path, '/')) {
      [$namespace, $name] = explode('/', $path);
    }
    if (!$gitlab_project_id && !$name) {
      return NULL;
    }

    $query = $this->connection
      ->select('drupalorg_project_repositories', 'pr')
      ->fields('pr', ['drupal_project_nid']);
    if ($gitlab_project_id) {
      $query->condition('pr.gitlab_project_id', $gitlab_project_id);
    }
    else {
      $query->condition('pr.gitlab_namespace', $namespace);
      $query->condition('pr.gitlab_project_name', $name);
    }
    $project = $query->execute()->fetchCol();
    if (count($project) === 1) {
      return $this->entityTypeManager->getStorage('node')->load($project[0]);
    }

    return NULL;
  }

  /**
   * Return the types of node that are considered projects.
   *
   * @return array
   *   Types of project.
   */
  public function projectNodeTypes(): array {
    return self::PROJECT_TYPES;
  }

  /**
   * Return whether a node is a project type or not.
   *
   * @param \Drupal\node\NodeInterface $node
   *   Node to check.
   *
   * @return bool
   *   True if it's a project, false otherwise.
   */
  public function isProject(NodeInterface $node): bool {
    return in_array($node->bundle(), $this->projectNodeTypes());
  }

  /**
   * Determine if a given node is a sandbox or full project.
   *
   * @param \Drupal\node\NodeInterface $node
   *   A fully-loaded node object representing the project.
   *
   * @return bool
   *   TRUE if the given node is a sandbox project, FALSE if full.
   */
  public function isSandbox(NodeInterface $node): bool {
    if ($node->hasField('field_project_type')) {
      $project_type = $node->get('field_project_type')->value ?? NULL;
      return $project_type === 'sandbox';
    }
    return FALSE;
  }

  /**
   * Get “branches” for a project.
   *
   * Each branch identifier is everything up to and including the final dot in
   * the version number.
   *
   * @param \Drupal\node\NodeInterface $project_node
   *   Project node.
   * @param bool $only_supported
   *   Only return the supported branches if TRUE, otherwise all.
   *
   * @return \stdClass[]
   *   Array keyed by branch.
   *
   * @throws \Exception
   */
  public function getVersions(NodeInterface $project_node, $only_supported = FALSE): array {
    $query = $this->connection->select('drupalorg_project_release_supported_versions', 'prsv')
      ->fields('prsv', ['branch', 'supported', 'recommended_release', 'latest_release', 'latest_security_release'])
      ->condition('prsv.nid', $project_node->id());
    if ($only_supported) {
      $query->condition('prsv.supported', TRUE);
    }
    return $query->execute()->fetchAllAssoc('branch');
  }

  /**
   * Update the project logo URL, if needed.
   *
   * @param \Drupal\node\NodeInterface $project_node
   *   Project node.
   * @param string $avatar_url
   *   'avatar_url' value from GitLab project API.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function updateLogo(NodeInterface $project_node, string $avatar_url): void {
    $existing_logo = $project_node->get('field_logo_url')->uri;
    if ($existing_logo != $avatar_url) {
      $this->logger->notice('Updating @project logo from "@existing_logo" → "@to"', [
        '@project' => $project_node->get('field_project_machine_name')->value,
        '@existing_logo' => $existing_logo ?? '',
        '@to' => $avatar_url,
      ]);
      $project_node->set('field_logo_url', $avatar_url)->save();
    }
  }

  /**
   * Get project type information.
   *
   * @param \Drupal\node\NodeInterface $project_node
   *   Project node.
   *
   * @return array
   *   Associative array with 'singular', 'plural', and 'url' keys.
   */
  public function getProjectType(NodeInterface $project_node): array {
    $default_url = Url::fromUserInput('/project/' . $project_node->bundle());
    switch ($project_node->bundle()) {
      case 'project_general':
        if (in_array('drupal-recipe', $project_node->get('field_project_composer_types')->value)) {
          return [
            'singular' => t('Recipe'),
            'plural' => t('Recipes'),
            'url' => Url::fromUserInput('/browse/recipes'),
          ];
        }
        else {
          return [
            'singular' => t('General project'),
            'plural' => t('General projects'),
            'url' => $default_url,
          ];
        }

      case 'project_module':
        return [
          'singular' => 'Module',
          'plural' => 'Modules',
          'url' => $default_url,
        ];

      case 'project_theme':
        return [
          'singular' => 'Theme',
          'plural' => 'Themes',
          'url' => $default_url,
        ];

      case 'project_distribution':
        return [
          'singular' => 'Distribution',
          'plural' => 'Distributions',
          'url' => $default_url,
        ];

      case 'project_theme_engine':
        return [
          'singular' => 'Theme engine',
          'plural' => 'Theme engines',
          'url' => $default_url,
        ];

      case 'project_drupalorg':
        return [
          'singular' => 'Community project',
          'plural' => 'Community projects',
          'url' => $default_url,
        ];

      case 'project_translation':
        return [
          'singular' => 'Translation',
          'plural' => 'Translations',
          'url' => $default_url,
        ];

      case 'project_core':
        return [
          'singular' => 'Drupal Core',
          'plural' => 'Drupal Cores',
          'url' => $default_url,
        ];
    }

    throw new \Exception('The entity is not a project.');
  }

}
