<?php

namespace Drupal\drupalorg\Traits;

use Drupal\user\Entity\User;
use Gitlab\Client;

/**
 * Helper function to get a Gitlab client instance.
 *
 * @todo Use https://git.drupalcode.org/project/gitlab_api instead.
 */
trait GitLabClientTrait {

  /**
   * Return a gitlab client instance.
   *
   * @return \Gitlab\Client
   *   Client instance.
   */
  protected function getGitLabClient(): Client {
    $config = \Drupal::config('drupalorg.gitlab_settings');
    if (
      empty($config->get('host')) ||
      empty($config->get('token')) ||
      $config->get('token') === 'CHANGE-ME'
    ) {
      throw new \Exception('GitLab host or token not set.');
    }
    $gitlab_client = new Client();
    $gitlab_client->setUrl($config->get('host'));
    $gitlab_client->authenticate($config->get('token'), Client::AUTH_HTTP_TOKEN);

    return $gitlab_client;
  }

  /**
   * Returns the gitlab user id for the given user if found.
   *
   * @param int $user_id
   *   Id of the user.
   *
   * @return int|null
   *   Id of the user in gitlab or null.
   */
  protected function getGitLabUserIdFromUserId(int $user_id) {
    $cache_id = 'gitlab_user_id_for_drupal_user_' . $user_id;
    if ($cached_data = \Drupal::cache()->get($cache_id)) {
      return $cached_data->data;
    }

    $user_gitlab_id = NULL;
    $client = $this->getGitLabClient();
    $drupal_user = User::load($user_id);
    $gitlab_username = $drupal_user->field_git_username->getValue()[0]['value'] ?? FALSE;
    if (empty($gitlab_username)) {
      return NULL;
    }
    else {
      try {
        $gitlab_users = $client->users()->all([
          'username' => $gitlab_username,
        ]);
        if (count($gitlab_users) == 1 && $gitlab_users[0]['username'] == $gitlab_username) {
          $user_gitlab_id = $gitlab_users[0]['id'];
          \Drupal::cache()->set($cache_id, $user_gitlab_id);
        }
      }
      catch (\Throwable $e) {
        \Drupal::logger('drupalorg')->error('Error searching users. Error: @error', [
          '@error' => $e->getMessage(),
        ]);
      }
    }

    return $user_gitlab_id;
  }

  /**
   * Returns the URL of the instance of GitLab in use.
   *
   * @return string
   *   URL of the instance.
   */
  public function getGitLabUrl(): string {
    return \Drupal::config('drupalorg.gitlab_settings')->get('host');
  }

}
