<?php

namespace Drupal\drupalorg;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\node\NodeInterface;
use Drupal\user\UserInterface;

/**
 * User related helper methods.
 */
class UserService {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $connection;

  /**
   * An instance of the entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Project service.
   *
   * @var \Drupal\drupalorg\ProjectService
   */
  protected ProjectService $projectService;

  /**
   * Construct method.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager service.
   * @param \Drupal\drupalorg\ProjectService $projectService
   *   Project service.
   */
  public function __construct(Connection $connection, EntityTypeManagerInterface $entityTypeManager, ProjectService $projectService) {
    $this->connection = $connection;
    $this->entityTypeManager = $entityTypeManager;
    $this->projectService = $projectService;
  }

  /**
   * Retrieve user by username.
   *
   * @param string $username
   *   Drupal username.
   *
   * @return \Drupal\user\UserInterface|null
   *   User entity or null.
   */
  public function getUserByUsername(string $username): ?UserInterface {
    $users = $this->getUsersByUsername([$username]);
    return !empty($users) ? array_shift($users) : NULL;
  }

  /**
   * Retrieve users by name.
   *
   * @param string[] $usernames
   *   List of usernames.
   *
   * @return \Drupal\user\UserInterface[]|null
   *   User entity or null.
   */
  public function getUsersByUsername(array $usernames): ?array {
    return $this->getUsersBy('name', $usernames);
  }

  /**
   * Retrieve user by git username.
   *
   * @param string $username
   *   Git username.
   *
   * @return \Drupal\user\UserInterface|null
   *   User entity or null.
   */
  public function getUserByGitUsername(string $username): ?UserInterface {
    $users = $this->getUsersByGitUsername([$username]);
    return !empty($users) ? array_shift($users) : NULL;
  }

  /**
   * Retrieve users by git username.
   *
   * @param string[] $usernames
   *   List of git usernames.
   *
   * @return \Drupal\user\UserInterface[]|null
   *   User entity or null.
   */
  public function getUsersByGitUsername(array $usernames): ?array {
    return $this->getUsersBy('field_git_username', $usernames);
  }

  /**
   * Generic function to get users by a certain value on a given field.
   *
   * @param string $field
   *   Name of the field.
   * @param array $values
   *   List of values for the field.
   *
   * @return \Drupal\user\UserInterface[]|null
   *   List of users matching the criteria or null.
   */
  protected function getUsersBy($field, array $values): ?array {
    $ids = $this->entityTypeManager->getStorage('user')->getQuery()
      ->accessCheck(FALSE)
      ->condition($field, $values, 'IN')
      ->execute();
    if (!empty($ids)) {
      return $this->entityTypeManager->getStorage('user')->loadMultiple($ids);
    }

    return NULL;
  }

  /**
   * List of node types that shared accounts can use.
   *
   * @return string[]
   *   List of node types.
   */
  public function sharedAccountAllowedNodeTypes() {
    $project_types = $this->projectService->projectNodeTypes();
    return array_merge(['organization', 'casestudy', 'event'], $project_types);
  }

  /**
   * Returns whether a given user account is shared.
   *
   * @param \Drupal\user\UserInterface $user
   *   User to check.
   *
   * @return bool
   *   Account is a shared one or not.
   */
  public function isSharedAccount(UserInterface $user) {
    return (bool) $user->get('field_shared_account_for_an_org')->value;
  }

  /**
   * Checks whether a user is shared and can access a certain node for editing.
   *
   * @param \Drupal\user\UserInterface $user
   *   User to check.
   * @param \Drupal\node\NodeInterface $node
   *   Node to check.
   *
   * @return bool
   *   Whether the user can edit the node or not.
   */
  public function isAccountAllowedToEditNode(UserInterface $user, NodeInterface $node) {
    if (
      $this->isSharedAccount($user) &&
      !in_array($node->bundle(), $this->sharedAccountAllowedNodeTypes())
    ) {
      return FALSE;
    }

    return TRUE;
  }

}
